 /*

	**********************************************Licensing **********************************************
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <https://www.gnu.org/licenses/>.


	********************************************Sketch Purpose********************************************
	This sketch is for the development of a multifunctional darkroom timer
	which incorporates timer functions and enlarger timer based mainly on the
	f-stop system.

	Change Log:
	Ver. 0.1ALPHA - Initial development. Core functions developed
	Ver. 0.1BETA - Menu system/UI/UX revamp and standardisation. Memory utilisation optimisation. Code cleanup.

	Codes from the following sites was used and all credit goes to the original author.
		Rotary interrupt - https://www.instructables.com/id/Improved-Arduino-Rotary-Encoder-Reading/
		ISR Debounce - https://forum.arduino.cc/index.php?topic=45000.0
    PinChangeInt.h - https://github.com/GreyGnome/PinChangeInt
*/



// ----------------------------------------------------------------------------------------------------- //

// Library to be included in sketch
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <Keypad.h>
#include <Math.h>
//#include <EEPROM.h> //Not yet in use as of ver1.1.0 
#include <PinChangeInt.h> // PinChange Interrupt library - https://github.com/GreyGnome/PinChangeInt
//#include <avr/interrupt.h> //Maybe remove if no required
LiquidCrystal_I2C lcd(0x27,20,4);

// Defines for keypad size and configuration. Test of your keypad to the pins should be done before commiting to final setup (i.e. soldered)
const byte ROWS = 4;
const byte COLS = 4;

char hexaKeys[ROWS][COLS] = {
	{'1', '2', '3', 'A'},
	{'4', '5', '6', 'B'},
	{'7', '8', '9', 'C'},
	{'*', '0', '#', 'D'}
};

byte rowPins[ROWS] = {12, 11, 10, 9};  //Change this section as required if different keypad wiring is used in your keypad
byte colPins[COLS] = {8, 7, 6, 5};  //Change this section as required if different keypad wiring is used in your keypad
Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);
char customKey = customKeypad.getKey(); //defines customKeypad to enable input through out the code
float temp = 0; //generic temporary store for inputs

// This section sets up the rotary encoder on D2 and D3 with interrupts to minimise load. Potentially change the volatile data to byte to save space
static byte pinA = 2; // Our first hardware interrupt pin is digital pin 2
static byte pinB = 3; // Our second hardware interrupt pin is digital pin 3
static byte pinC = 4; // Rotary push button on pin 4
volatile byte aFlag = 0; // let's us know when we're expecting a rising edge on pinA to signal that the encoder has arrived at a detent
volatile byte bFlag = 0; // let's us know when we're expecting a rising edge on pinB to signal that the encoder has arrived at a detent (opposite direction to when aFlag is set)
volatile byte encoderPos = 0; //this variable stores our current value of encoder position. Change to byte or uin16_t instead of byte if you want to record a larger range than 0-255
volatile byte oldEncPos = 0; //stores the last encoder position value so we can compare to the current reading and see if it has changed (so we know when to print to the serial monitor)
volatile byte reading = 0; //somewhere to store the direct values we read from our interrupt pins before checking to see if we have moved a whole detent

// Setup for push switchs on pin 13 and 15
const byte startstopfoot = 13;
const byte enlargerfocus = 15;
byte startstopfootState = LOW;
byte enlargerfocusState = LOW;

// Setup for relay pins
const byte enlargerRelay = 16; //used to be in1
const byte safelightRelay = 17; //used to be in2
byte enlargerState = LOW; //turn enlarger off on start
byte safelightState = LOW;  //turn safelight on on start

//Setup for buzzer data pin
const byte buzzer = 14; //set buzzer to pin 14

//Setup for menu function
byte mainFunction = 10; //used to select primary function i.e. timer, test strip, etc. Starts with function 10 which does not have any function assigned
int menuSelection = 0; //to detect which menu is selected. Do not let menuselection <= 0
byte menuSelectionL2 = 1; //to detect which (sub)menu is selected. Do not let menuselection <= 0
byte menuSelectionL3 = 1; //to detect which (sub)menu is selected. Do not let menuselection <= 0
byte menuSelectionState = HIGH;  //to detect pushbutton on rotary, rotary is NC switch i.e. pressing the button equals to a LOW and not high
byte menuSelectionStateL2 = HIGH;
byte menuSelectionAwareness = 0;
byte menuSelectionMode = 0; //determine if menu selection 0 (no changing) 1 (changing mode)
byte menuChangemode = 0; //To determine when rotary button is pushed shoud it enter into menu change mode (0) or menu confirmation (1)
int vposition = 0; //used for verticle selection of menu
byte blinkerStart = 0;
byte blinkerNow = 0;
byte blinkerStartInt = 0;
byte blinkerNowInt = 0;
byte blinkerState = 0;
byte blinkerOld = 0;

// Variable used for millis debounce (https://www.brainy-bits.com/arduino-switch-debounce/)
//byte TimeOfLastDebounce = 0;  // holds the last time the switch was pressed
//byte DelayofDebounce = 100;  // amount of time that needs to be experied between presses

byte selectorL[8] = //icon creation for menu arrow pointing left 
{
	B00010,
	B00110,
	B01110,
	B11110,
	B01110,
	B00110,
	B00010,
	B00000
};

byte selectorR[8] = //icon creation for menu arrow pointing right
{
	B01000,
	B01100,
	B01110,
	B01111,
	B01110,
	B01100,
	B01000,
	B00000
};

// Setup for timer function variables - used also in other modes. mainFunction = 1
int i1 = 0; // data for converted miliseconds to seconds
int m1 = 0; // data for converted seconds to minutes
int s1 = 0; // data to show current seconds of the minute
int s1Old = 0; //capture the last second
byte alarmrang = 0; // data to capture that buzzer buzzed
float a1; // start point for timer
float c1; //current point for timer
float e1; //end point for timer
byte st1=0; // state change - 0-reset, 1-running, 2-stopped
byte clockMode=1; //clock mode selection
int countDownTime = 0; //countdown time to be counted from i.e. 60 means 60 minutes countdown

//Setup for Test strip mode variables
int stepsC = 5;
float Stops = 3;
float firststep = 2;
int stripmode = 1; // Change to byte to minimise memory use
int executeMode = 0;
int stopMode = 0;

//Setup for enlarger timer mode variables
int enlargermode = 1; //choice between Absolute or Addition mode
int zones = 3; //number of dodge burn zones
float dryDown = 0; //% dry down to be applied as a discount to all zones steps
float filterFactor = 1; //multiplier for filters of rating 4 or higher multiplied to the zone step time
int zoneEntrypos = 0; //to select which part of the program to edit
float zonesStops[25]; //input for stops value per zone - define new byte data to enable quick change number of zones
int dodgeBurn[25]={1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1}; //2 for dodge(or minus) 1 for burn (to add)
float filterUsed[25]; //input for filter value to be used per zone
float zonesTime[25]; //output for stops in seconds
float zonesTimeExec[25]; //output in seconds to run enlarger
float totalDodgeTime = 0; //total time dodge being applied

//byte FgradeChar1[25];
//byte FgradeChar2[25];

void setup() {

	//LCD setup for 20x4 with backlight always on
 lcd.init();
lcd.backlight();
	//Setups the rotary encoder for interrupt
	pinMode(pinA, INPUT_PULLUP); // set pinA as an input, pulled HIGH to the logic voltage (5V or 3.3V for most cases)
	pinMode(pinB, INPUT_PULLUP); // set pinB as an input, pulled HIGH to the logic voltage (5V or 3.3V for most cases)
	pinMode(pinC, INPUT_PULLUP); // designate rotary encoder push button for pin change intterupt input
	attachInterrupt(0,PinA,RISING); // set an interrupt on PinA, looking for a rising edge signal and executing the "PinA" Interrupt Service Routine (below)
	attachInterrupt(1,PinB,RISING); // set an interrupt on PinB, looking for a rising edge signal and executing the "PinB" Interrupt Service Routine (below)
// 	attachPinChangeInterrupt (pinC, menuChanging, FALLING);

	//Setup for push switchs on pin 13 and 15
	pinMode (startstopfoot, INPUT_PULLUP);
	pinMode (enlargerfocus, INPUT_PULLUP);
	//No interrupt required for startstopfoot switch
	attachPinChangeInterrupt(enlargerfocus,enlargerTurnOnOff,FALLING);

	pinMode(buzzer, OUTPUT); //Setup for buzzer

	//Setup for relay
	pinMode (enlargerRelay, OUTPUT);
	digitalWrite (enlargerRelay, LOW);
	pinMode (safelightRelay, OUTPUT);
	digitalWrite (safelightRelay, HIGH);
	enlargerState = LOW; //turn enlarger off on start
	safelightState = HIGH;  //turn safelight on on start - just change this at initiation

	lcd.clear();
}


void loop() {
	if (mainFunction == 10) { //mainFunction 10 use as indicator of first run
		lcd.setCursor(0,0);
		lcd.print("Darkroom Timer");
		lcd.setCursor(0,1);
		lcd.print("Ver. 1.1.1");
		lcd.setCursor(0,3);
		lcd.print("Press 'A' to start");

	}
 
	if (mainFunction != 10) {  //mainFunction 10 use as indicator of first run
		lcd.clear();
	}

	customKey = customKeypad.getKey();  //Look for inputs to move into mainFunction 0
	switch(customKey){
	case 'A':
		mainFunction = 0;
		menuSelection = 0;
		lcd.clear();
	}

        menuSelectionState = digitalRead(pinC); //monitor rotary button to move into mainFunction 0
      if (menuSelectionState == LOW) {
    menuChangemode = 1;
    mainFunction = 0;
    menuSelection = 0;
    lcd.clear();
      }

            while (menuSelectionState == LOW && menuChangemode == 1) {
        delay(10);
        menuSelectionState = digitalRead(pinC);
      }

      if (menuSelectionState == HIGH && menuChangemode == 1) {
        menuChangemode = 0;
        menuSelectionState = digitalRead(pinC);
      }

      
	while (mainFunction == 0){ //Menu selection mode
		
		menuSelectionMode=1; //Become a modechange state
		menuSelectionState = HIGH;

		while (menuSelectionMode == 1) {
			lcd.setCursor (2,0);
			lcd.print ("Clock & Timer");
			lcd.setCursor (2,1);
			lcd.print ("Test Strip");
			lcd.setCursor (2,2);
			lcd.print ("Enlarger Timer");
			lcd.setCursor (1,menuSelection);
			lcd.write(2);
      			
			if (oldEncPos != encoderPos){ //Allows selection of editing with rotary selector
				menuSelection = menuSelection + (encoderPos - oldEncPos);
				if (menuSelection > 2){  //recheck if this is >3 is better or more accurate
					menuSelection = 0;
				}
				if (menuSelection < 0){  //recheck if this is <1 is better or more accurate
					menuSelection = 2;
				}
				oldEncPos = encoderPos;
				lcd.clear();
			}

			menuSelectionState = digitalRead(pinC); //monitor rotary button for confirmation to exit mode
			if (menuSelectionState == LOW && menuSelectionMode == 1) {
				mainFunction = menuSelection+1;
				menuChangemode == 1;
				delay(100);
				menuSelectionState = HIGH;
				menuChangemode = 0;
				menuSelectionMode = 0;
				menuChangemode = 0;
				clockMode = 1; //required as menu reset clockMode was put as 9 to enable the mainFunction loop to exit
				lcd.clear();
			}
		}
	}

	while (mainFunction == 1){ //Clock & Timer mode

		lcd.setCursor (0,0);
		lcd.print ("Timer");
		lcd.setCursor (0,1);
		lcd.print ("Mode");
		lcd.setCursor (7,0);
		if (clockMode == 1) {
			lcd.print ("Minute Tick");
			lcd.setCursor(7,1);
			lcd.print("            ");
		}
		if (clockMode == 2) {
			lcd.print ("Second Tick");
			lcd.setCursor(7,1);
			lcd.print("            ");
		}  
		if (clockMode == 3) {
			lcd.print ("Countdown  "); 
			lcd.setCursor(7,1);
			lcd.print(countDownTime);
//			lcd.setCursor(9,1);
			lcd.print(" Min Count"); 
		}		
		lcd.setCursor (6,vposition);
		lcd.write(2);
		
		if (clockMode == 3 && oldEncPos != encoderPos){ //Allows selection of editing with rotary selector if clock function is 3/Countdown
			vposition = vposition + (encoderPos - oldEncPos);
			if (vposition > 1){
				vposition = 0;
			}
			if (vposition < 0){
				vposition = 1;
			}
			oldEncPos = encoderPos;
			lcd.clear();
		}
   


		customKey = customKeypad.getKey();  //Look for inputs
		if (vposition == 0 && customKey){  //Change the clock modes
			if (customKey == '1' || customKey == '2' || customKey == '3'){  //Monitors for mode change
				lcd.clear();
				clockMode = customKey - '0';
			}
		}

returnToMenu();
    
		if (vposition == 1){ //Look for ability to enter minutes for countdown mode

			switch (customKey){
			case '0' ... '9':
				temp = temp * 10 + (customKey - '0');
				break;
				
			case '#':
     if (temp > 999) {
      temp = 999;
     }
				countDownTime = temp;
				temp = 0;
       m1 = countDownTime;
       e1 = countDownTime*60;
      // lcd.clear();
				break;
			}
		}		
		
		// Timer running and printing mode       
		lcd.setCursor(7,2);
		lcd.print(m1);
//		lcd.setCursor(9,2);
		lcd.print(" Min ");
		lcd.print(s1);
//		lcd.setCursor(16,2);
		lcd.print(" Sec  ");
		lcd.setCursor(7,3);
		if (st1==0) {
			lcd.print("READY");
		}
		if (st1==1) {
			lcd.print("RUNNING");
		}
		if (st1==2) {
			lcd.print("STOPPED");
		}
		
		
		startstopfootState = digitalRead(startstopfoot); //Monitors for start of clock run
		if (startstopfootState == LOW && clockMode != 3) {   //monitors for start stop of timer if clockMode is not 3/countdown mode
			if (st1==0) // if state is reset (0) and button is pressed go to state 1 (running) and mark current time in a1
			{
				st1=1;
				a1=millis();
				s1Old=s1;
				delay(50);
//				lcd.clear();
			}
			else if (st1==1) // if state is running (1) and button is pressed go to state 2 (stopped)
			{
				st1=2;
				delay(250);
			}
			else if (st1==2) // start timer 2 // if state is stopped and button is pressed go to state 0 (reset)
			{
				st1=0;
				lcd.clear();
				delay(150);
			}
		}
		
		else if (startstopfootState == LOW && clockMode == 3) {   //monitors for start stop of timer if clockMode is 3/countdown mode
			if (st1==0) // if state is reset (0) and button is pressed go to state 1 (running) and mark current time in a1
			{
				st1=1;
				a1=millis();
				s1Old=s1;
				delay(250);
				//lcd.clear();
			}
			else if (st1==1) // if state is running (1) and button is pressed go to state 2 (stopped)
			{
				st1=2;
				delay(250);
       noTone(buzzer);
			}
			else if (st1==2) // start timer 2 // if state is stopped and button is pressed go to state 0 (reset)
			{
				st1=0;
				lcd.clear();
				delay(250);
			}
			//lcd.clear();
		}
		
		
		//Section for Minute and Second ticker
		if (st1==0 && clockMode != 3) // if state is reset (0) park current and start timer point (a and c) as 0.
		{
			c1 = 0;
			a1 = 0;
			i1 = (c1 - a1) / 1000;
			m1 = i1 / 60;
			s1 = i1-(m1*60);
		};
		
		if (st1==1 && clockMode != 3) //if state is running (1) current timer point is constantly updated with timer. difference between current running time and start timer point gives time since state change
		{
			c1 = millis(); //updates current time since state change
			i1 = (c1 - a1) / 1000; //calculate time since state change
			m1 = i1 / 60; //calculate time in minutes that has ecliped
			s1 = i1-(m1*60); //calculates time in seconds since last minute has ecliped
		};
		
		if (st1==2 && clockMode != 3) // if state is stopped (2) current timer point (c) is no longer updated but calculation to show time difference between current and start timer points continues (c1 - a1)
		{
			i1 = (c1 - a1) / 1000;
			m1 = i1 / 60;
			s1 = i1-(m1*60);
		};
		
		
		//Section for countdown timer
		if (st1==0 && clockMode == 3) // if state is reset (0) park current and start timer point (a and c) as 0.
		{
			c1 = 0;
			a1 = 0;
			i1 = 0;
			s1 = 0;
		};
		
		if (st1==1 && clockMode == 3) //if state is running (1) current timer point is constantly updated with timer. difference between current running time and start timer point gives time since state change
		{
			c1 = millis(); //updates current time since state change
			i1 = (c1 - a1) / 1000; //calculate time in seconds since state change
			m1 = (e1 - i1) / 60; //calculate time in minutes that has ecliped
			s1 = (e1 - i1) - m1*60;//calculates time in seconds since last minute has ecliped
			
		}
		
		if (st1==2 && clockMode == 3) // if state is stopped (2) current timer point (c) is no longer updated but calculation to show time difference between current and start timer points continues (c1 - a1)
		{
			i1 = (c1 - a1) / 1000; //calculate time in seconds since state change
			m1 = (e1 - i1) / 60; //calculate time in minutes that has ecliped
			s1 = (e1 - i1) - m1*60;//calculates time in seconds since last minute has ecliped     
		}
		
		
		//Section for Minute and Second ticker buzzer
		if (clockMode == 1) {
			if ((st1==1 && s1==0 && alarmrang==0)) //buzzer to ring at 0 sec
			{
        beepTwice();
				lcd.clear();
				alarmrang = 1;
			}
			if ((st1==1 && s1==50 && alarmrang==0)) //buzzer to ring at 50 sec
			{
        beepOnce();
				lcd.clear();
				alarmrang = 1;
			}
			if (alarmrang==1 && (s1==1 || s1==51)) //buzzer ringing monitoring so that only rings once
			{
				alarmrang = 0;
			}
		}
		
		if (clockMode == 2) {
			if ((st1==1 && s1!=s1Old)) //if timer is running then buzz every time s1 increases...
			{
				beepOnce();
				s1Old=s1;
			}
			
		}
		
		if (clockMode == 3) {
			if (st1 == 1 && s1 == 0 && m1 == 10) {
beepLong();
			}				
			
			if (st1 == 1 && s1 <= 60 && s1 >= 0 && m1 == 0  && s1 != s1Old) {
				beepOnce();
				s1Old=s1;
			}
			
			if (st1 == 1 && s1 == 0 && m1 == 0) {
beepxLong();
			}
		}
		

	}

	while (mainFunction == 2){ //Test strip mode
		
		lcd.setCursor (0,0);
		lcd.print ("Test");
		lcd.setCursor (0,1);
		lcd.print ("Strip");
		lcd.setCursor (0,2);
		lcd.print ("Mode");
		lcd.setCursor (0,3);
		lcd.print ("Setup");
		lcd.setCursor (7,0);
		lcd.print ("Mode :");
		lcd.setCursor (14,0);
		if (stripmode == 1){
			lcd.print ("Cover");
		}
		else if (stripmode == 2){
			lcd.print ("Open");
		}  

		lcd.setCursor (7,1);
		lcd.print ("Steps:");
		lcd.setCursor (14,1);
		lcd.print (stepsC);

		lcd.setCursor (7,2);
		lcd.print ("Start:");
		lcd.setCursor (14,2);
		lcd.print (firststep);
		lcd.setCursor (9,2);

		lcd.setCursor (7,3);
		lcd.print ("FStop:");
		lcd.setCursor (14,3);
		lcd.print (Stops);

		lcd.setCursor (19,vposition);
		lcd.write(1);

		if (oldEncPos != encoderPos){ //Allows selection of editing with rotary selector
			vposition = vposition + (encoderPos - oldEncPos);
			if (vposition > 3){
				vposition = 0;
			}
			if (vposition < 0){
				vposition = 3;
			}
			oldEncPos = encoderPos;
			lcd.clear();
		}


		customKey = customKeypad.getKey();  //Look for inputs

		if (vposition == 0 && customKey){  //Change Open or Cover mode of test strip
			if (customKey == '1' || customKey == '2'){  //Monitors for mode change
				lcd.clear();
				stripmode = customKey - '0';
			}
		}

		if (vposition == 1){  //Change the number of steps - now maximum is 9 steps
			switch(customKey){
			case '0'...'9':
				lcd.clear();
				stepsC = customKey - '0';
			}
		}

		if (vposition == 2){ //https://www.youtube.com/watch?v=CSXGS1J3iZ4  changes start time

			switch (customKey){
			case '0' ... '9':
				temp = temp * 10 + (customKey - '0');
				break;
				
			case '#':
				lcd.clear();
				firststep = temp/100;
				temp = 0;
				break;
			}
		}

		if (vposition == 3){ //changes how many fstops between the first and last step in the test strip i.e. 5 steps and 5 stops equals to 1 stop difference per step
			switch(customKey){
			case '0'...'9':
				temp = temp * 10 + (customKey - '0');
				break;
				
			case '#':
				lcd.clear();
				Stops = temp/100;
				temp = 0;
				break;
			}
		}


		if (customKey == 'C'){  //Monitors for execute mode
			executeMode = 1;
			lcd.clear();
		} 

returnToMenu(); //Monitor for request to return to Main menu

		while (executeMode == 1){
			lcd.setCursor (0,0);
			lcd.print ("Test");
			lcd.setCursor (0,1);
			lcd.print ("Strip");
			lcd.setCursor (0,2);
			lcd.print ("Mode");
			lcd.setCursor (0,3);
			lcd.print ("Exec.");
			lcd.setCursor (7,0);
			byte calculate = 0;

			if (stripmode == 1 && executeMode == 1 && calculate == 0){
				float Fstopsteps =  Stops / (stepsC-1);
				float strips[stepsC];
				float firststeproot = log(firststep)/log(2);
				
				for (byte i=1; i<stepsC+1; i=i+1){
					strips[i-1] = pow(2,(firststeproot+(Fstopsteps*(i-1))));
				}
				float exposure[stepsC];
				exposure[0] = firststep;
				for (byte i=1; i<stepsC+1; i=i+1){
					exposure[i] = strips[i]-strips[i-1];
				}

				calculate = 1; //calculation of exposure and storage into array complete

				for (byte a=1; a<stepsC+1; ){
					lcd.setCursor (8,0);
					lcd.print ("Close Strip");
					lcd.setCursor (8,1);
					lcd.print (a);
					lcd.setCursor (10,1);
					lcd.print ("of");
					lcd.setCursor (13,1);
					lcd.print (stepsC);
					lcd.setCursor (8,2);
					lcd.print (exposure[a-1]);
//					lcd.setCursor (14 ,2);
					lcd.print (" Secs");
					lcd.setCursor (8,3);
					lcd.print ("Ready");

					customKey = customKeypad.getKey();  // look for exit mode input
					switch (customKey){
					case 'B':
						executeMode = 0;
						a = stepsC + 2;
						lcd.clear();
						break;
					}

					startstopfootState = digitalRead(startstopfoot);
					if (startstopfootState == LOW){
						digitalWrite (enlargerRelay, HIGH);
						digitalWrite (safelightRelay, LOW);
						double temp1 = 0;
						a1 = millis();
						e1 = (exposure[a-1]*1000 + a1);
						lcd.clear();
						
						while (e1 > millis()) {
							temp1 = ((e1 - millis())/1000);
							lcd.setCursor (8,2);
							lcd.print (temp1);
//							lcd.setCursor (14,2);
							lcd.print (" Secs   ");
							lcd.setCursor (8,3);
							lcd.print ("Run");
						}

						digitalWrite (enlargerRelay, LOW);
						digitalWrite (safelightRelay, HIGH);
						
						startstopfootState = digitalRead(startstopfoot);
						while (startstopfootState == LOW){
							startstopfootState = digitalRead(startstopfoot);
							lcd.setCursor (8,3);
							lcd.print ("Release");//do nothing
						}
						lcd.clear();
						a = a+1;
						if (a == stepsC+1){
							executeMode = 0;
							lcd.clear();
							lcd.setCursor (9,3);
							lcd.print ("Complete");
							delay(5000);
							lcd.clear();
							break;
						}
					}
					
				}
				break;    
			}

			
			if (stripmode == 2 && executeMode == 1 && calculate == 0){
				float Fstopsteps =  Stops / (stepsC-1);
				float strips[stepsC];
				float firststeproot = log(firststep)/log(2);
				
				for (byte i=1; i<stepsC+1; i=i+1){
					strips[i-1] = pow(2,(firststeproot+(Fstopsteps*(i-1))));
				}
				float exposure[stepsC];
				exposure[0] = firststep;
				for (byte i=1; i<stepsC+1; i=i+1){
					exposure[i] = strips[i]-strips[i-1];
				}
				calculate = 1;
				byte b=1;

				for (byte a=stepsC; a>-1; ){
					lcd.setCursor (8,0);
					lcd.print ("Expose Strip");
					lcd.setCursor (8,1);
					lcd.print (b);
					lcd.setCursor (10,1);
					lcd.print ("of");
					lcd.setCursor (13,1);
					lcd.print (stepsC);
					lcd.setCursor (8,2);
					lcd.print (exposure[a-1]);
//					lcd.setCursor (14 ,2);
					lcd.print (" secs");
					lcd.setCursor (8,3);
					lcd.print ("Ready");
					
					customKey = customKeypad.getKey();
					if (customKey == 'B'){  //Monitors for exit execute mode
						executeMode = 0;
            lcd.clear();
						break;
					}
					startstopfootState = digitalRead(startstopfoot);
					if (startstopfootState == LOW){
						digitalWrite (enlargerRelay, HIGH);
						digitalWrite (safelightRelay, LOW);
            double temp1 = 0;
            a1 = millis();
            e1 = (exposure[a-1]*1000 + a1);
            lcd.clear();
						
            while (e1 > millis()) {
              temp1 = ((e1 - millis())/1000);
              lcd.setCursor (8,2);
              lcd.print (temp1);
//              lcd.setCursor (14,2);
              lcd.print (" Secs   ");
              lcd.setCursor (8,3);
              lcd.print ("Run");
            }
            
//            lcd.setCursor (9,3);
//						lcd.print ("Run");
//						delay(exposure[a-1]*1000);
						digitalWrite (enlargerRelay, LOW);
						digitalWrite (safelightRelay, HIGH);

            startstopfootState = digitalRead(startstopfoot);
            while (startstopfootState == LOW){
              startstopfootState = digitalRead(startstopfoot);
              lcd.setCursor (8,3);
              lcd.print ("Release");//do nothing
            }
            lcd.clear();            
						a=a-1;
						b=b+1;
						if (b == stepsC+1){
							executeMode = 0;
							lcd.clear();
							lcd.setCursor (9,3);
							lcd.print ("Complete");
							delay(5000);
							lcd.clear();
							b = 1;
							break;
						}
					}
				}
				break;    
			}
		}
	}

	while (mainFunction == 3){ //Enlarger timer mode
		byte calculate = 0;

		lcd.setCursor (0,0);
		lcd.print ("Enlrg");
		lcd.setCursor (0,1);
		lcd.print ("Timer");
		lcd.setCursor (0,2);
		lcd.print ("Mode");
		lcd.setCursor (0,3);
		lcd.print ("Setup");

		lcd.setCursor (6,0);
		if (enlargermode == 1){
			lcd.print ("Absolute"); //Absolute mode means knowing all the different exposure zone values and letting the system calculate the exposure
		}
		else if (enlargermode == 2){
			lcd.print ("Addition"); //Addition mode means building up on continous exposure - i.e. you have precalculated the differnce in zones and are exposing only on the diffience
		}  

		lcd.setCursor (6,1);
		lcd.print (zones);
		lcd.setCursor (9,1);
		lcd.print ("Zones");

		lcd.setCursor (6,2);
		lcd.print (dryDown);
		lcd.setCursor (11,2);
		lcd.print ("% DryDown");

		lcd.setCursor (6,3);
		lcd.print (filterFactor);
		lcd.setCursor (9,3);
		lcd.print ("x Filter >4");
		
		lcd.setCursor (5,vposition);
		lcd.write(2);

		if (oldEncPos != encoderPos){ //Allows selection of editing with rotary selector
			vposition = vposition + (encoderPos - oldEncPos);
			if (vposition > 3){ 
				vposition = 0;
			}
			if (vposition < 0){
				vposition = 3; 
			}
			oldEncPos = encoderPos;
			lcd.clear();
		}

		customKey = customKeypad.getKey();  //Look for inputs
		if (vposition == 0){
			switch (customKey){
			case '1' ... '2':
				enlargermode = customKey - '0';
				break;
			}
		}

		if (vposition == 1){
			switch(customKey){
			case '0'...'9':
				temp = temp * 10 + (customKey - '0');
				break;
				
			case '#':
				if (temp > 25) { //sets the max number of zones for enlargers to 25... technically able to support up to 90 zones but practically does not make sense
					temp = 25;
				}
				zones = temp;
				temp = 0;
				break;
			}
		}

		if (vposition == 2){
			switch(customKey){
			case '0'...'9':
				temp = temp * 10 + (customKey - '0');
				break;
				
			case '#':
				dryDown = temp/100;
				temp = 0;
				break;
			}
		}

		if (vposition == 3){
			switch(customKey){
			case '0'...'9':
				temp = temp * 10 + (customKey - '0');
				break;
				
			case '#':
				filterFactor = temp/10;
				temp = 0;
			}
		}

		switch(customKey){
		case 'C':
			executeMode = 1;
			lcd.clear();
			vposition = 0;
			break;
		}

returnToMenu();

		if (executeMode == 1) {
			lcd.clear();
		}

		while (executeMode == 1) {

			lcd.setCursor (0,0);
			lcd.print ("Enlrg");
			lcd.setCursor (0,1);
			lcd.print ("Timer");
			lcd.setCursor (0,2);
			lcd.print ("Mode");
			lcd.setCursor (0,3);
			lcd.print ("Prog.");
			
			lcd.setCursor (6,0);
			lcd.print ("Zone ");
//			lcd.setCursor (11,0);
			lcd.print (zoneEntrypos+1);
//			lcd.setCursor (13,0);
			lcd.print (" of ");
//			lcd.setCursor (16,0);
			lcd.print (zones);

			lcd.setCursor (6,1);
			if (dodgeBurn[zoneEntrypos] == 1){
				lcd.print ("Burn ");
			}
			if (dodgeBurn[zoneEntrypos] == 2){
				lcd.print ("Dodge ");
			}
			
//			lcd.setCursor (12,1);
			lcd.print (zonesTime[zoneEntrypos]);
			lcd.print((char)042);
			

			lcd.setCursor (6,2);
			if (dodgeBurn[zoneEntrypos] == 1){
				lcd.print ("+");
			}
			else if (dodgeBurn[zoneEntrypos] == 2){
				lcd.print ("-");
			}      
			lcd.print (zonesStops[zoneEntrypos]);
			lcd.print (" EV");      
			
			lcd.setCursor (6,3);
			if (filterUsed[zoneEntrypos] > 0) {         //Filters between 0.5 and 5.0
        lcd.print (filterUsed[zoneEntrypos]);
        lcd.setCursor (9,3);
//        lcd.print (".");
//        lcd.print (FgradeChar2[zoneEntrypos]);       
			}
      else if (filterUsed[zoneEntrypos] == 0) {   //Filter 0
        lcd.print ("  0");
//        lcd.print (FgradeChar2[zoneEntrypos]);
      }
      else if (filterUsed[zoneEntrypos] == -1) {  //Filter 00
        lcd.print (" 00");
//        lcd.print (FgradeChar2[zoneEntrypos]);
      }
      else if (filterUsed[zoneEntrypos] == -2) {  //No Filter
        lcd.print ("No");
      }      
			lcd.print (" Filter");

			
			lcd.setCursor (5,vposition);
			lcd.write(2);

			if (oldEncPos != encoderPos){ //Allows selection of editing with rotary selector
				vposition = vposition + (encoderPos - oldEncPos);
				if (vposition > 3){
					vposition = 0;
				}
				if (vposition < 0){
					vposition = 3;
				}
				oldEncPos = encoderPos;
				lcd.clear();
			}
			
			customKey = customKeypad.getKey();  
			if (vposition == 0){ //Entry to change which zone to edit
				switch (customKey){
				case '0'... '9':
					temp = temp * 10 + (customKey - '0');
					break;
					
				case '#':
					if (temp > zones) { //prevent from entering zone higher than defined
						temp = zones;
					}
					if (temp < 1) {  //prevent from entering zone 0
						temp = 1;
					}              
					lcd.clear();
					zoneEntrypos = temp - 1;
					temp = 0;
					break;
				}
			}
			
			if (vposition == 1){ //Entry to change between Burn and Dodge
				switch(customKey){
				case '1'...'2':
					dodgeBurn[zoneEntrypos] = (customKey - '0');
					lcd.clear();
					break;
				}
			}
			
			if (vposition == 2){ //Entry to change which zone EV/Stops to expose
				switch(customKey){
				case '0'...'9':
					temp =  (temp * 10 + (customKey - '0'));
					break;
					
				case '#':
					lcd.clear();
					zonesStops[zoneEntrypos] = temp/100;
					zonesTime[zoneEntrypos] = pow(2, zonesStops[zoneEntrypos]);  //calculates and put into array the time in seconds for the zone
					temp = 0;
					break;
				}
			}

			if (vposition == 3){ //Entry to change filter factor
        switch(customKey){
        case '0'...'9':
          temp = temp * 10 + (customKey - '0');
            break;
                    
        case '#':
          lcd.clear();
          temp = temp/10;

          if (temp >= 0.5 && temp <= 5) {           //Filters between 0.5 and 5.0
            temp=(0.5*(round(temp*2)));
//            FgradeChar1[zoneEntrypos] = temp;
//            FgradeChar2[zoneEntrypos] = ((temp - FgradeChar1[zoneEntrypos])*10);
            filterUsed[zoneEntrypos] = temp;
          }          
          else if (temp >= 0.01 && temp <=0.49) {  //Filter 0
            filterUsed[zoneEntrypos] = 0;
//            FgradeChar1[zoneEntrypos] = 0;
 //           FgradeChar2[zoneEntrypos] = 0;
          }
          else if (temp == 0) {                    //Filter 00
            filterUsed[zoneEntrypos] = -1;
//            FgradeChar1[zoneEntrypos] = 0;
//            FgradeChar2[zoneEntrypos] = 0;
          }
          else if (temp > 5) {                    //No Filter
            filterUsed[zoneEntrypos] = -2;
//            FgradeChar1[zoneEntrypos] = 0;
 //           FgradeChar2[zoneEntrypos] = 0;
          }
//                            lcd.setCursor(14,2);
//                  lcd.print(filterUsed[zoneEntrypos]);                   
          temp = 0;
          break;
        }
			}

			switch(customKey){
			case 'C':
				totalDodgeTime = 0;
				for (byte a = 0; a < zones; a++){ //this is to calculate total dodge time to be taken out of base exposure
					if (dodgeBurn[a] == 2) {
						totalDodgeTime = totalDodgeTime + zonesTime[a];
					}
				}    

				if (enlargermode == 1 && calculate == 0) {
					for (byte a = 0; a < zones; a++){ //this is to calculate all burn times by minusing total dodge time
						if (dodgeBurn[a] == 1) {
							zonesTimeExec[a] = (zonesTime[a] - totalDodgeTime);
                if (zonesTimeExec[a] <= 0) {
                    lcd.setCursor (6,0);
                    lcd.print ("Zone ");
                    lcd.print (a+1);
                    lcd.print (" of ");
                    lcd.print (zones);
                    lcd.setCursor (6,1);
                    lcd.print ("is less than");
                    lcd.setCursor (6,2);
                    lcd.print ("total dodge");
                    zonesTimeExec[a] = 0;
                    beepTwice();
                    delay(1500);
                }
						}
						if (dodgeBurn[a] == 2) {
							zonesTimeExec[a] = zonesTime[a];
						}        
					}

					for (byte a = 0; a < zones; a++){ //adds the filter factor multiplier to filters more than 4
						if (filterFactor >= 1 && filterUsed[a] >= 4 && filterUsed[a] <= 5) {
							zonesTimeExec[a] = zonesTimeExec[a]*filterFactor;
						}
					}

					for (byte a = 0; a < zones; a++){ //applies drydown "discount" to all exposures
						zonesTimeExec[a] = zonesTimeExec[a]*((100-dryDown)/100);
					}
					calculate = 1;             
				}

				if (enlargermode == 2 && calculate == 0) {
					for (byte a = 0; a < zones; a++){ //since user inputed the expected exposure time; pass data to execute array
						zonesTimeExec[a] = zonesTime[a];        
					}

					for (byte a = 0; a < zones; a++){
						if (filterFactor >= 1 && filterUsed[a] >= 4) { //adds the filter factor multiplier to filters more than 4
							zonesTimeExec[a] = zonesTimeExec[a]*filterFactor;
						}
					}

					for (byte a = 0; a < zones; a++){  //applies drydown "discount" to all exposures
						zonesTimeExec[a] = zonesTimeExec[a]*((100-dryDown)/100);
					}
					calculate = 1;             
				}

				executeMode = 2;
				lcd.clear();
				vposition = 0;
				break;

			}

returnToMenu();

			if (customKey == 'B'){ //try to figure out how to exit programming mode back to setup. reprogram all * and B to align between entering value and back.
				executeMode = 0;
				lcd.clear();
				break;
			}
		}

		while (executeMode == 2) {
			
			lcd.setCursor (0,0);
			lcd.print ("Enlrg");
			lcd.setCursor (0,1);
			lcd.print ("Timer");
			lcd.setCursor (0,2);
			lcd.print ("Mode");
			lcd.setCursor (0,3);
			lcd.print ("Exec.");  


			for (byte a=0; a<zones; ){
				
				lcd.setCursor (6,0);
				lcd.print ("Zone");
				lcd.setCursor (11,0);
				lcd.print (a+1);
				lcd.print (" of ");
				lcd.print (zones);

				lcd.setCursor (6,1);
				if (dodgeBurn[a] == 1){
					lcd.print ("Burn");
				}
				if (dodgeBurn[a] == 2){
					lcd.print ("Dodge");
				}
				
				lcd.setCursor (12,1);
				lcd.print (zonesTimeExec[a]);
				lcd.print((char)042);
				

				lcd.setCursor (6,2);
				if (dodgeBurn[a] == 1){
					lcd.print ("+");
				}
				else if (dodgeBurn[a] == 2){
					lcd.print ("-");
				}      
				lcd.print (zonesStops[a]); // this part is wrong?? not the same as the exec time
				lcd.setCursor (12,2);
				lcd.print ("EV");      
				
				lcd.setCursor (6,3);
				lcd.print ("Filter ");

      if (filterUsed[a] > 0) {         //Filters between 0.5 and 5.0
        lcd.print (filterUsed[a]);
        lcd.setCursor (16,3);
        lcd.print ("    ");
      }
      else if (filterUsed[a] == 0) {   //Filter 0
        lcd.print ("  0");
      }
      else if (filterUsed[a] == -1) {  //Filter 00
        lcd.print (" 00");
      }
      else if (filterUsed[a] == -2) {  //No Filter
        lcd.print ("None");
      }      


				
				lcd.setCursor (15,2);
				lcd.print ("Ready");
        
        customKey = customKeypad.getKey(); 
				      if (customKey == 'B'){ //return to previous menu
                executeMode = 1;
                lcd.clear();
                break;
              }
              
				startstopfootState = digitalRead(startstopfoot);
				if (startstopfootState == LOW){
					digitalWrite (enlargerRelay, HIGH);
					digitalWrite (safelightRelay, LOW);
					double temp1 = 0;
					a1 = millis();
					e1 = (zonesTimeExec[a]*1000 + a1);
					lcd.clear();
					
					while (e1 > millis()) {
						temp1 = (e1 - millis())/1000;
						lcd.setCursor (8,1);
						lcd.print (temp1);
						lcd.print(" Secs   ");
						lcd.setCursor (8,2);
						lcd.print ("Run");
            if (temp1 <= 3.01 && temp1 >= 2.99 || temp1 <= 2.01 && temp1 >=1.99 || temp1 <= 1.01 && temp1 >= 0.99) { //to beep in the last 3 seconds to indicate near end of exposure
              beepShort();
            }
            if (temp1 <= 0.01 && temp1 >= 0) { //to beep in the end of exposure
              beepLong();
            }
					}

					digitalWrite (enlargerRelay, LOW);
					digitalWrite (safelightRelay, HIGH);
          
					startstopfootState = digitalRead(startstopfoot);
          
					while (startstopfootState == LOW){
						startstopfootState = digitalRead(startstopfoot);
						//lcd.clear();
						lcd.setCursor (8,3);
						lcd.print ("Release");//do nothing
					}
					
					lcd.clear();
					a = a+1;
					if (a == zones){
						executeMode = 1;
						lcd.clear();
						lcd.setCursor (9,3);
						lcd.print ("Complete");
//            beepLong();
						delay(4000);
						lcd.clear();
						break;
					}
				}
			}


			if (customKey == 'B'){  //Monitors for exit execute mode
				executeMode = 1;
			}  
		}
	}

	while (mainFunction == 4){ //Not in use



	}


} //End of void loop









// ----------------------------------------------------------------------------------------------------- //
// -----------------------------------This section is for custom functions------------------------------ //



void PinA(){ //to monitor rotary position
	cli(); //stop interrupts happening before we read pin values
	reading = PIND & 0xC; // read all eight pin values then strip away all but pinA and pinB's values
	if(reading == B00001100 && aFlag) { //check that we have both pins at detent (HIGH) and that we are expecting detent on this pin's rising edge
		encoderPos --; //decrement the encoder's position count
		bFlag = 0; //reset flags for the next turn
		aFlag = 0; //reset flags for the next turn
	}
	else if (reading == B00000100) bFlag = 1; //signal that we're expecting pinB to signal the transition to detent from free rotation
	sei(); //restart interrupts
}

void PinB(){ //to monitor rotary position2
	cli(); //stop interrupts happening before we read pin values
	reading = PIND & 0xC; //read all eight pin values then strip away all but pinA and pinB's values
	if (reading == B00001100 && bFlag) { //check that we have both pins at detent (HIGH) and that we are expecting detent on this pin's rising edge
		encoderPos ++; //increment the encoder's position count
		bFlag = 0; //reset flags for the next turn
		aFlag = 0; //reset flags for the next turn
	}
	else if (reading == B00001000) aFlag = 1; //signal that we're expecting pinA to signal the transition to detent from free rotation
	sei(); //restart interrupts
}

void enlargerTurnOnOff() {
	static unsigned long last_interrupt_time = 0;
	unsigned long interrupt_time = millis();
	// If interrupts come faster than 20ms, assume it's a bounce and ignore
	if (interrupt_time - last_interrupt_time > 20)
	{
		if (enlargerState == LOW) {
			digitalWrite(enlargerRelay,HIGH);
			digitalWrite(safelightRelay,LOW);
			enlargerState = HIGH;
			safelightState = LOW;
			delay(100);
		}

		else if (enlargerState == HIGH) {
			digitalWrite(enlargerRelay,LOW);
			digitalWrite(safelightRelay,HIGH);
			enlargerState = LOW;
			safelightState = HIGH;
			delay(100);
		}
		last_interrupt_time = interrupt_time;
	}
}

void beepOnce() {  //custom function to beep once
	tone(buzzer, 2500);     //start sound...
	delay(100);
	noTone(buzzer);     // Stop sound...
}

void beepTwice() {  //custom function to beep once
        tone(buzzer,2500); // Send 2.5KHz sound signal...
        delay(100);
        noTone(buzzer);     // Stop sound...
        delay(250);
        tone(buzzer,2500); // Send 2.5KHz sound signal...
        delay(100);
        noTone(buzzer);     // Stop sound...
}

void beepLong() {
        tone(buzzer,2500); // Send 2.5KHz sound signal...
        delay(1000);
        noTone(buzzer);     // Stop sound...
}

void beepxLong() {
        tone(buzzer,2500); // Send 2.5KHz sound signal...
}

void beepShort() {
        tone(buzzer,2500); // Send 2.5KHz sound signal...
        delay(100);
        noTone(buzzer);     // Stop sound...
}

void returnToMenu() {
//	customKey = customKeypad.getKey();  //Look for inputs to move into mainFunction 0
	switch(customKey){
	case 'A':
		mainFunction = 0;
		menuSelection = 0;
		clockMode = 9;
    vposition = 0;
		lcd.clear();
	}
}

void saveAllData() {
	
}
